"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.configFileReaderFactory = exports.ConfigFileReader = void 0;
const dotenv = require("dotenv");
const file_stream_service_1 = require("../io/file-stream.service");
const config_file_1 = require("./config-file");
dotenv.config();
class ConfigFileReader {
    constructor(fileStream) {
        this.fileStream = fileStream;
    }
    readConfig(workingDirectory, configFilePath, dbUrl) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!configFilePath) {
                throw new Error(`Missing config command line argument`);
            }
            const configFile = yield this.fileStream.readFileAsJsonAsync(configFilePath);
            this.validate(configFile, dbUrl);
            return configFile;
        });
    }
    validate(configFile, fallbackDbUrl = null) {
        if (!configFile) {
            throw new Error(`Missing config file ${configFile}`);
        }
        configFile.databaseUrl = this.getDbUrl(configFile, fallbackDbUrl);
        if (!configFile.databaseUrl) {
            throw new Error(`Missing database url, Valid options IConfigFile.dotEnvDatabaseUrlKey | IConfigFile.databaseUrl | argv.dbUrl`);
        }
        if (!configFile.tasks || !Array.isArray(configFile.tasks)) {
            throw new Error(`Missing IConfigFile.tasks`);
        }
        this.validateTasks(configFile);
    }
    getDbUrl(configFile, fallbackDbUrl) {
        if (configFile.databaseUrl) {
            return configFile.databaseUrl;
        }
        if (configFile.dotEnvDatabaseUrlKey) {
            return process.env[configFile.dotEnvDatabaseUrlKey];
        }
        return fallbackDbUrl;
    }
    validateTasks(configFile) {
        let taskIdx = 0;
        for (const task of configFile.tasks) {
            if (!config_file_1.ConfigTaskTypes[task.type]) {
                throw new Error(`tasks[${taskIdx}].type must be one of the following ${config_file_1.ConfigTaskTypes.ExportOrRunTablesAsSeed}, ${config_file_1.ConfigTaskTypes.ExportTablesAsCodeFile}`);
            }
            if (!task.label) {
                throw new Error(`tasks[${taskIdx}].label must be a distinct string in the tasks list`);
            }
            if (!task.outDir) {
                throw new Error(`tasks[${taskIdx}].outDir must be a string`);
            }
            if (!task.tables || !Array.isArray(task.tables)) {
                throw new Error(`tasks[${taskIdx}].tables must be an array of type (TableSeedDef | TableCodeFileDef)[]`);
            }
            if (task.type === config_file_1.ConfigTaskTypes.ExportTablesAsCodeFile) {
                this.validateCodeFileTables(task, taskIdx);
            }
            if (task.type === config_file_1.ConfigTaskTypes.ExportOrRunTablesAsSeed) {
                if (task.excludeColumns && !Array.isArray(task.excludeColumns)) {
                    throw new Error(`tasks[${taskIdx}].excludeColumns must be a string[]`);
                }
                this.validateExportOrRunTables(task, taskIdx);
            }
            taskIdx++;
        }
    }
    validateCodeFileTables(task, taskIdx) {
        let tableIdx = 0;
        for (const table of task.tables) {
            if (!table.entityName) {
                throw new Error(`tasks[${taskIdx}].tables[${tableIdx}].entityName must be a string`);
            }
            if (!table.keyColumn) {
                throw new Error(`tasks[${taskIdx}].tables[${tableIdx}].keyColumn must be a string`);
            }
            if (!table.tableName) {
                throw new Error(`tasks[${taskIdx}].tables[${tableIdx}].tableName must be a string`);
            }
            if (!table.valueColumn) {
                throw new Error(`tasks[${taskIdx}].tables[${tableIdx}].valueColumn must be a string`);
            }
            tableIdx++;
        }
    }
    validateExportOrRunTables(task, taskIdx) {
        let tableIdx = 0;
        for (const table of task.tables) {
            if (!table.tableName) {
                throw new Error(`tasks[${taskIdx}].tables[${tableIdx}].tableName must be a string`);
            }
            if (!table.primaryKeyNames || !Array.isArray(table.primaryKeyNames)) {
                throw new Error(`tasks[${taskIdx}].tables[${tableIdx}].valueColumn must be a string[]`);
            }
            if (table.excludeColumns && !Array.isArray(table.excludeColumns)) {
                throw new Error(`tasks[${taskIdx}].tables[${tableIdx}].excludeColumns must be a string[]`);
            }
            if (table.overrideColumns && typeof table.overrideColumns !== 'object') {
                throw new Error(`tasks[${taskIdx}].tables[${tableIdx}].overrideColumns must be a {[key: string]: any} key value pair object`);
            }
            tableIdx++;
        }
    }
}
exports.ConfigFileReader = ConfigFileReader;
exports.configFileReaderFactory = {
    createInstance: (loggerInstance) => {
        const fileStreamService = new file_stream_service_1.FileStreamService(loggerInstance);
        return new ConfigFileReader(fileStreamService);
    },
};
//# sourceMappingURL=config-file-reader.js.map