"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TableToCoderWriter = void 0;
const lodash_1 = require("lodash");
const file_writer_1 = require("./file-writer");
class TableToCoderWriter {
    constructor(outDir, logger, seedQueryRunner, fileStream) {
        this.outDir = outDir;
        this.logger = logger;
        this.seedQueryRunner = seedQueryRunner;
        this.fileStream = fileStream;
    }
    writeCodeFile(seedData) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.info(`Exporting table ${seedData.tableName}`, TableToCoderWriter.name);
            let tableRowData = yield this.seedQueryRunner.getRawTableData(seedData.tableName);
            tableRowData = (0, lodash_1.sortBy)(tableRowData, [seedData.valueColumn, seedData.keyColumn]);
            this.logger.info(`Found in table table ${seedData.tableName} ${tableRowData.length} rows`, TableToCoderWriter.name);
            this.logger.info(`Writing file ${seedData.entityName}-seed.json in directory ${this.outDir}`, TableToCoderWriter.name);
            const codeFileName = `${seedData.entityName}Values`;
            const fileWriter = new file_writer_1.FileWriter(this.fileStream, this.outDir, `${codeFileName}.ts`);
            const TypescriptKeyType = `${seedData.entityName}Key`;
            const TypescriptKeyId = `${seedData.entityName}Id`;
            fileWriter.wl(`export type ${TypescriptKeyType} =`);
            tableRowData.forEach((row, index) => fileWriter.wl(`${fileWriter.tab}| '${row[seedData.keyColumn]}'${index === tableRowData.length - 1 ? ';' : ''}`));
            fileWriter.wl('');
            fileWriter.wl(`export type ${TypescriptKeyId} =`);
            tableRowData.forEach((row, index) => fileWriter.wl(`${fileWriter.tab}| ${row[seedData.valueColumn]}${index === tableRowData.length - 1 ? ';' : ''}`));
            fileWriter.wl('');
            const interfaceName = `${codeFileName}Interface`;
            fileWriter.wl(`interface ${interfaceName} {`);
            tableRowData.forEach((row) => fileWriter.wl(`${fileWriter.tab}${this.getKeyValue(row[seedData.keyColumn])}: ${this.getValue(row[seedData.valueColumn])};`));
            fileWriter.wl(`${fileWriter.tab}ValueToKey: Map<${TypescriptKeyId}, ${TypescriptKeyType}>;`);
            fileWriter.wl(`${fileWriter.tab}KeyToValue: Map<${TypescriptKeyType}, ${TypescriptKeyId}>;`);
            fileWriter.wl(`${fileWriter.tab}AllValues: () => ${TypescriptKeyId}[];`);
            fileWriter.wl(`${fileWriter.tab}AllKeys: () => ${TypescriptKeyType}[];`);
            fileWriter.wl(`}`);
            fileWriter.wl('');
            fileWriter.wl(`export const ${codeFileName}:  ${interfaceName} = {`);
            tableRowData.forEach((row) => {
                fileWriter.wl(`${fileWriter.tab}${this.getKeyValue(row[seedData.keyColumn])}: ${this.getValue(row[seedData.valueColumn])},`);
            });
            this.writeValueToKeyMap(fileWriter, tableRowData, seedData);
            this.writeKeyToValueMap(fileWriter, tableRowData, seedData);
            this.writeAllValuesArray(fileWriter, tableRowData, seedData);
            this.writeAllKeysArray(fileWriter, tableRowData, seedData);
            fileWriter.wl(`};`);
            yield fileWriter.saveToDir();
        });
    }
    getKeyValue(value) {
        const valueToCheck = value ? value.toString().trim().replace(' ', '').replace('-', '') : '';
        return valueToCheck;
    }
    getValue(value) {
        if (typeof value === 'string') {
            return `'${value}'`;
        }
        if (typeof value === 'number') {
            return value;
        }
        return value;
    }
    writeKeyToValueMap(fileWriter, tableRowData, seedData) {
        const entries = tableRowData.map((row) => {
            return [this.getValue(row[seedData.keyColumn]), this.getValue(row[seedData.valueColumn])];
        });
        this.writeMap(fileWriter, 'KeyToValue', entries);
    }
    writeValueToKeyMap(fileWriter, tableRowData, seedData) {
        const entries = tableRowData.map((row) => {
            return [this.getValue(row[seedData.valueColumn]), this.getValue(row[seedData.keyColumn])];
        });
        this.writeMap(fileWriter, 'ValueToKey', entries);
    }
    writeAllValuesArray(fileWriter, tableRowData, seedData) {
        this.writeGetArray(fileWriter, 'AllValues', tableRowData.map((row) => this.getValue(row[seedData.valueColumn])));
    }
    writeAllKeysArray(fileWriter, tableRowData, seedData) {
        this.writeGetArray(fileWriter, 'AllKeys', tableRowData.map((row) => this.getValue(row[seedData.keyColumn])));
    }
    writeMap(fileWriter, variableName, entries) {
        fileWriter.wl(`${fileWriter.tab}${variableName}: new Map([`);
        entries.forEach((row) => {
            fileWriter.wl(`${fileWriter.tab}${fileWriter.tab}[${row[0]}, ${row[1]}],`);
        });
        fileWriter.wl(`${fileWriter.tab}]),`);
    }
    writeGetArray(fileWriter, variableName, entries) {
        fileWriter.wl(`${fileWriter.tab}${variableName}: () => {`);
        fileWriter.wl(`${fileWriter.tab}${fileWriter.tab}return [`);
        entries.forEach((value) => {
            fileWriter.wl(`${fileWriter.tab}${fileWriter.tab}${fileWriter.tab}${value},`);
        });
        fileWriter.wl(`${fileWriter.tab}${fileWriter.tab}];`);
        fileWriter.wl(`${fileWriter.tab}},`);
    }
}
exports.TableToCoderWriter = TableToCoderWriter;
//# sourceMappingURL=table-to-coder-writer.js.map