"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PostgresSeedQueryRunner = void 0;
const convert_to_array_1 = require("../config-file/convert-to-array");
const date_extensions_1 = require("../date-extensions");
class PostgresSeedQueryRunner {
    constructor(queryRunner, logger) {
        this.queryRunner = queryRunner;
        this.logger = logger;
        if (!this.queryRunner) {
            throw new Error('private readonly queryRunner: QueryRunner is Null');
        }
        if (!this.logger) {
            throw new Error('private readonly logger: ISeedDataLogger is Null');
        }
    }
    createTable(table) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.queryRunner.createTable(table, true);
        });
    }
    getColumnValue(newRow, colname) {
        let columnValue = newRow[colname] === undefined ? null : newRow[colname];
        if (typeof columnValue === 'string' && this.isDateMatch(columnValue)) {
            columnValue = columnValue.replace(`new DATE(`, '').replace(')', '');
            columnValue = new Date(date_extensions_1.DateExtensions.convertUTCDateToLocalDate(columnValue));
        }
        else if (columnValue && !(columnValue instanceof Date) && typeof columnValue === 'object') {
            columnValue = JSON.stringify(columnValue);
        }
        return columnValue;
    }
    transaction(runInTransaction) {
        return __awaiter(this, void 0, void 0, function* () {
            const shouldStartTransaction = !this.queryRunner.isTransactionActive;
            try {
                if (shouldStartTransaction) {
                    yield this.queryRunner.startTransaction();
                }
                yield runInTransaction();
                if (shouldStartTransaction) {
                    yield this.queryRunner.commitTransaction();
                }
            }
            catch (e) {
                console.log(e);
                this.logger.debug(e.message, PostgresSeedQueryRunner.name);
                if (shouldStartTransaction) {
                    yield this.queryRunner.rollbackTransaction();
                }
                throw e;
            }
        });
    }
    getRawTableData(tableName) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.queryRunner.query(`SELECT * FROM ${tableName}`);
        });
    }
    getRow(tableName, query) {
        return __awaiter(this, void 0, void 0, function* () {
            const whereQuery = [];
            const whereParams = [];
            let i = 1;
            for (let colname in query) {
                if (query[colname] === null) {
                    whereQuery.push(`("${colname}" IS NULL)`);
                }
                else {
                    whereQuery.push(`("${colname}" = ${'$' + i})`);
                    whereParams.push(query[colname]);
                    i++;
                }
            }
            const data = yield this.queryRunner.query(`SELECT * FROM ${tableName} where ${whereQuery.join(' AND ')}`, whereParams);
            if (Array.isArray(data)) {
                return data[0];
            }
            return null;
        });
    }
    insertRow(tableName, newRowData) {
        return __awaiter(this, void 0, void 0, function* () {
            const insertColumns = [];
            const insertValues = [];
            const insertParams = [];
            let i = 1;
            for (const colname in newRowData) {
                let columnValue = this.getColumnValue(newRowData, colname);
                insertColumns.push(`"${colname}"`);
                insertValues.push(`${'$' + i}`);
                insertParams.push(columnValue);
                i++;
            }
            const queryInput = `INSERT INTO "${tableName}" (${insertColumns.join(', ')}) VALUES (${insertValues.join(', ')})`;
            this.logger.debug(`${queryInput} params: [${insertParams}]`, PostgresSeedQueryRunner.name);
            yield this.queryRunner.query(queryInput, insertParams);
        });
    }
    updateRow(tableName, newRowData, matchRowQuery) {
        return __awaiter(this, void 0, void 0, function* () {
            const logSetQuery = [];
            const updateSetQuery = [];
            const updateWhereQuery = [];
            const updateParams = [];
            let paramIndex = 1;
            for (const colname in newRowData) {
                let columnValue = this.getColumnValue(newRowData, colname);
                updateSetQuery.push(`"${colname}" = ${'$' + paramIndex}`);
                updateParams.push(columnValue);
                logSetQuery.push(`"${colname}" = ${newRowData[colname]}`);
                paramIndex++;
            }
            for (const colname in matchRowQuery) {
                if (matchRowQuery[colname] === null) {
                    updateWhereQuery.push(`("${colname}" IS NULL)`);
                }
                else {
                    updateWhereQuery.push(`("${colname}" = ${'$' + paramIndex})`);
                    updateParams.push(matchRowQuery[colname]);
                    paramIndex++;
                }
            }
            const queryInfo = `UPDATE "${tableName}" SET ${updateSetQuery.join(', ')} WHERE ${updateWhereQuery.join(' AND ')}`;
            this.logger.debug(`${queryInfo} params [${updateParams}]`, PostgresSeedQueryRunner.name);
            yield this.queryRunner.query(queryInfo, updateParams);
        });
    }
    deleteRow(tableName, where) {
        return __awaiter(this, void 0, void 0, function* () {
            const deleteQuery = [];
            const deleteParams = [];
            let i = 1;
            for (const colname in where) {
                if (where[colname] === null) {
                    deleteQuery.push(`("${colname}" IS NULL)`);
                }
                else {
                    deleteQuery.push(`("${colname}" = ${'$' + i})`);
                    deleteParams.push(this.getColumnValue(where, colname));
                    i++;
                }
            }
            const queryInfo = `DELETE FROM "${tableName}" WHERE ${deleteQuery.join(' AND ')}`;
            this.logger.debug(`${queryInfo} params: ${deleteParams}`, PostgresSeedQueryRunner.name);
            yield this.queryRunner.query(queryInfo, deleteParams);
        });
    }
    insertSeedRow(dbRow, seedData, excludedColumnNames) {
        return __awaiter(this, void 0, void 0, function* () {
            const newRowData = {};
            for (const colname in dbRow) {
                if (excludedColumnNames.indexOf(colname) === -1) {
                    newRowData[colname] = dbRow[colname];
                }
            }
            yield this.insertRow(seedData.tableName, dbRow);
        });
    }
    updateSeedRow(dbRow, newRow, seedData, excludedColumnNames) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const newRowData = {};
            const excludedColumns = [...(0, convert_to_array_1.convertToArray)(excludedColumnNames), ...(0, convert_to_array_1.convertToArray)(seedData.excludeColumns)];
            for (const colname in dbRow) {
                if (excludedColumns.indexOf(colname) === -1) {
                    newRowData[colname] = newRow[colname];
                }
            }
            seedData.overrideColumns = (_a = seedData.overrideColumns) !== null && _a !== void 0 ? _a : {};
            for (const colname in seedData.overrideColumns) {
                newRowData[colname] = seedData.overrideColumns[colname];
            }
            const whereQuery = {};
            for (const colname of seedData.primaryKeyNames) {
                whereQuery[colname] = dbRow[colname];
            }
            yield this.updateRow(seedData.tableName, newRowData, whereQuery);
        });
    }
    deleteSeedRow(dbRow, seedData) {
        return __awaiter(this, void 0, void 0, function* () {
            const deleteWhere = {};
            for (const colname of seedData.primaryKeyNames) {
                deleteWhere[colname] = dbRow[colname];
            }
            yield this.deleteRow(seedData.tableName, deleteWhere);
        });
    }
    updateIdSequence(tableName) {
        return __awaiter(this, void 0, void 0, function* () {
            const hasSequanceTable = yield this.queryRunner.query(`SELECT relname FROM pg_class where relname = $1`, [`${tableName}_id_seq`]);
            if (!hasSequanceTable || (hasSequanceTable && !hasSequanceTable.length)) {
                return;
            }
            const maxTableValue = yield this.queryRunner.query(`SELECT MAX("id") FROM ${tableName}`);
            if (maxTableValue && maxTableValue.length) {
                this.logger.debug(`ALTER SEQUENCE ${tableName}_id_seq RESTART WITH ${maxTableValue[0].max + 1}`, PostgresSeedQueryRunner.name);
                yield this.queryRunner.query(`ALTER SEQUENCE ${tableName}_id_seq RESTART WITH ${maxTableValue[0].max + 1}`);
            }
        });
    }
    getTableColumns(tableName) {
        return __awaiter(this, void 0, void 0, function* () {
            const tableColumns = yield this.queryRunner.query(`SELECT column_name FROM information_schema.columns WHERE table_schema = 'public' AND table_name = $1`, [tableName]);
            return tableColumns.map((tc) => tc.column_name);
        });
    }
    isDateMatch(value) {
        return /^new DATE\(.+\)$/.test(value);
    }
}
exports.PostgresSeedQueryRunner = PostgresSeedQueryRunner;
//# sourceMappingURL=postgres-seed-query-runner.js.map