"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SeedDataRunner = void 0;
const crypto = require("crypto");
const lodash_1 = require("lodash");
const typeorm_1 = require("typeorm");
const convert_to_array_1 = require("../config-file/convert-to-array");
const date_extensions_1 = require("../date-extensions");
const table_to_coder_writer_1 = require("../io/table-to-coder-writer");
class SeedDataRunner {
    constructor(seedQueryRunner, fileStream, logger) {
        this.seedQueryRunner = seedQueryRunner;
        this.fileStream = fileStream;
        this.logger = logger;
        this.isInitialized = false;
        this.seedRunnerTableColumns = {
            seedFile: 'seed_file',
            seedFileHash: 'seed_file_hash',
        };
        this.seedRunnerTable = new typeorm_1.Table({
            name: 'seed_runner_history',
            columns: [
                new typeorm_1.TableColumn({
                    name: this.seedRunnerTableColumns.seedFile,
                    type: 'varchar(128)',
                    isUnique: true,
                    isNullable: false,
                }),
                new typeorm_1.TableColumn({
                    name: this.seedRunnerTableColumns.seedFileHash,
                    type: 'varchar(256)',
                    isUnique: true,
                    isNullable: false,
                }),
            ],
        });
        this.isReadyToRun = this.getIsInitialized();
    }
    exportTablesToCode(seedDataList, outDir) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.isReadyToRun;
            yield this.fileStream.rmDir(outDir);
            for (const seedData of seedDataList) {
                const tableToCodeFileWriter = new table_to_coder_writer_1.TableToCoderWriter(outDir, this.logger, this.seedQueryRunner, this.fileStream);
                yield tableToCodeFileWriter.writeCodeFile(seedData);
            }
        });
    }
    exportTablesAsSeed(task) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.isReadyToRun;
            yield this.fileStream.rmDir(task.outDir);
            for (const table of task.tables) {
                yield this.exportTableAsSeed(table, task);
            }
        });
    }
    exportTableAsSeed(table, task) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            yield this.isReadyToRun;
            let tableColumns = yield this.seedQueryRunner.getTableColumns(table.tableName);
            table.overrideColumns = (_a = table.overrideColumns) !== null && _a !== void 0 ? _a : {};
            const excludeColumns = [...(0, convert_to_array_1.convertToArray)(task.excludeColumns), ...(0, convert_to_array_1.convertToArray)(table.excludeColumns)];
            tableColumns = tableColumns.filter((c) => excludeColumns.indexOf(c) === -1);
            let fullTableData = yield this.seedQueryRunner.getRawTableData(table.tableName);
            let fullTableDataCopy = [];
            for (const row of fullTableData) {
                const newRow = {};
                for (const column of tableColumns) {
                    newRow[column] = row[column];
                    if (newRow[column] instanceof Date) {
                        newRow[column] = `new DATE(${date_extensions_1.DateExtensions.convertLocalDateToUTCDate(newRow[column])})`;
                    }
                }
                for (let column in table.overrideColumns) {
                    newRow[column] = table.overrideColumns[column];
                }
                fullTableDataCopy.push(newRow);
            }
            const dataToExport = (0, lodash_1.sortBy)(fullTableDataCopy, table.primaryKeyNames);
            yield this.fileStream.writeFileWithObject(task.outDir, `${table.tableName}.json`, dataToExport);
        });
    }
    runSeeds(seedDataList, excludedColumnNames, inputDir) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.seedQueryRunner.transaction(() => __awaiter(this, void 0, void 0, function* () {
                const changedSeeds = yield this.getChangedSeeds(seedDataList, inputDir);
                for (const changedFile of changedSeeds) {
                    this.logger.info(`Updating or Inserting seed data in table ${changedFile.seedData.tableName} `, SeedDataRunner.name);
                    yield this.updateOrInsertSeeds(changedFile, excludedColumnNames);
                }
                for (const seedData of [...changedSeeds].reverse()) {
                    this.logger.info(`Removing seed data which does not exist anymore in table ${seedData.seedData.tableName}`, SeedDataRunner.name);
                    yield this.deleteExtraSeeds(seedData);
                }
                for (const changedFile of changedSeeds) {
                    this.logger.info(`Saving new hash for table ${changedFile.seedData.tableName}`, SeedDataRunner.name);
                    yield this.updateOrInsertSeedHash(changedFile);
                }
            }));
        });
    }
    getChangedSeeds(seedDataList, inputDir) {
        return __awaiter(this, void 0, void 0, function* () {
            const outData = [];
            for (const seed of seedDataList) {
                const fileName = this.getFileName(seed.tableName);
                const filePath = this.fileStream.resolve([inputDir, fileName]);
                const fileContent = yield this.readFileAsString(filePath);
                const result = yield this.isFileUnChanged(this.getFileName(seed.tableName), fileContent);
                if (result.isFileChanged) {
                    outData.push({
                        fileContent: fileContent,
                        hash: result.currentHash,
                        databaseRow: result.databaseRow,
                        seedData: seed,
                    });
                }
                else {
                    this.logger.info(`Seed data for table ${seed.tableName} skipped: content unchanged`, SeedDataRunner.name);
                }
            }
            return outData;
        });
    }
    deleteExtraSeeds(changedFile) {
        return __awaiter(this, void 0, void 0, function* () {
            const seedMap = yield this.createDataByPrimaryKeyMap(changedFile.fileContent, changedFile.seedData);
            const tableRowData = yield this.seedQueryRunner.getRawTableData(changedFile.seedData.tableName);
            for (const dbRow of tableRowData) {
                const rowHashKey = this.tokenizePrimaryKey(dbRow, changedFile.seedData.primaryKeyNames);
                if (!seedMap.has(rowHashKey)) {
                    yield this.seedQueryRunner.deleteSeedRow(dbRow, changedFile.seedData);
                }
            }
        });
    }
    updateOrInsertSeeds(changedFile, excludedColumnNames) {
        return __awaiter(this, void 0, void 0, function* () {
            const seedData = changedFile.seedData;
            const seedMap = yield this.createDataByPrimaryKeyMap(changedFile.fileContent, seedData);
            const tableRowData = yield this.seedQueryRunner.getRawTableData(seedData.tableName);
            for (const dbRow of tableRowData) {
                const rowHashKey = this.tokenizePrimaryKey(dbRow, seedData.primaryKeyNames);
                this.logger.debug(`Updating row in table ${seedData.tableName}`, SeedDataRunner.name);
                const rowSeedData = seedMap.get(rowHashKey);
                if (!rowSeedData) {
                    continue;
                }
                yield this.seedQueryRunner.updateSeedRow(dbRow, rowSeedData, seedData, excludedColumnNames);
                seedMap.delete(rowHashKey);
            }
            const remainingValues = seedMap.entries();
            for (const [key, value] of remainingValues) {
                yield this.seedQueryRunner.insertSeedRow(value, seedData, excludedColumnNames);
            }
            const hasIdPrimaryKey = seedData.primaryKeyNames.find((it) => it === 'id');
            if (hasIdPrimaryKey) {
                yield this.seedQueryRunner.updateIdSequence(seedData.tableName);
            }
        });
    }
    createDataByPrimaryKeyMap(fileContent, seedData) {
        return __awaiter(this, void 0, void 0, function* () {
            const seedDataValues = JSON.parse(fileContent);
            const seedMap = seedDataValues.reduce((acc, item) => {
                const hashKey = this.tokenizePrimaryKey(item, seedData.primaryKeyNames);
                acc.set(hashKey, item);
                return acc;
            }, new Map());
            return seedMap;
        });
    }
    readFileAsString(inputFile) {
        return __awaiter(this, void 0, void 0, function* () {
            const inputBuffer = yield this.fileStream.readFileAsync(inputFile);
            return inputBuffer.toString();
        });
    }
    tokenizePrimaryKey(item, primaryKeyNames) {
        const tokenizedKey = [];
        for (const key of primaryKeyNames) {
            tokenizedKey.push(item[key]);
        }
        return tokenizedKey.join('_prkc_');
    }
    isFileUnChanged(fileName, fileContent) {
        return __awaiter(this, void 0, void 0, function* () {
            const newFileHash = yield this.hashFile(fileContent);
            const dbHashRow = yield this.seedQueryRunner.getRow(this.seedRunnerTable.name, {
                [this.seedRunnerTableColumns.seedFile]: fileName,
            });
            return {
                isFileChanged: !dbHashRow || dbHashRow[this.seedRunnerTableColumns.seedFileHash] !== newFileHash,
                databaseRow: dbHashRow,
                currentHash: newFileHash,
            };
        });
    }
    hashFile(fileContent) {
        return new Promise((resolve, reject) => {
            const hash = crypto.createHash('md5');
            hash.setEncoding('hex');
            hash.write(fileContent);
            hash.end();
            resolve(hash.read());
        });
    }
    getIsInitialized() {
        return new Promise((resolve, reject) => {
            if (this.isInitialized) {
                resolve(this.isInitialized);
            }
            this.initializeCacheTable()
                .then(() => {
                this.isInitialized = true;
                resolve(this.isInitialized);
            })
                .catch((e) => reject(e));
        });
    }
    initializeCacheTable() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.seedQueryRunner.createTable(this.seedRunnerTable);
        });
    }
    getFileName(data) {
        return `${data}.json`;
    }
    updateOrInsertSeedHash(changedFile) {
        return __awaiter(this, void 0, void 0, function* () {
            const newRowData = {
                [this.seedRunnerTableColumns.seedFile]: this.getFileName(changedFile.seedData.tableName),
                [this.seedRunnerTableColumns.seedFileHash]: changedFile.hash,
            };
            if (changedFile.databaseRow) {
                yield this.seedQueryRunner.updateRow(this.seedRunnerTable.name, newRowData, {
                    [this.seedRunnerTableColumns.seedFile]: this.getFileName(changedFile.seedData.tableName),
                });
                return;
            }
            yield this.seedQueryRunner.insertRow(this.seedRunnerTable.name, newRowData);
        });
    }
}
exports.SeedDataRunner = SeedDataRunner;
//# sourceMappingURL=seed-data-runner.js.map